/****************************************************************************

  DEVHOOK 0.4x FEP sample

****************************************************************************/

#include "common.h"

PSP_MODULE_INFO("PSP DEVHOOK 0.4x FEP", 0x1000, 1, 1);   // 0x1000 = Kernel MODE
PSP_MAIN_THREAD_ATTR(0); // 0 for kernel mode too

extern int sceKernelExitDeleteThread(int id);

/////////////////////////////////////////////////////////////////////////////
// Load/Start Direct
/////////////////////////////////////////////////////////////////////////////
static int load_start(const char *target_path)
{
  int result;
  int mid;

  // Direct target boot
  result = sceIoAssign("disc0:","umd0:","isofs:",1,0,0);
  Kprintf("----------ASSIGN disc0: result %08X\n",result);
  // result = 80020326 already mounted , OK?
//  if(result<0) return result;

  Kprintf("LoadTargetModule '%s':",target_path);
  mid = sceKernelLoadModule(target_path,0,0);
  Kprintf("%08X\n",mid);
  if(mid<0) return mid;

  Kprintf("Start Target Module:");
  result = sceKernelStartModule(mid,strlen(target_path)+1,(void *)target_path,0,0);
  Kprintf("%08X\n",mid);
  if(result<0) return mid;

  Kprintf("\n\n ---------- START TARGET MODULE ----------\n");
  return mid;
}

////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	struct SceKernelLoadExecParam loadparam;

	// SIO init for LOG
#if 1
	pspDebugSioInit();
	pspDebugSioSetBaud(115200);
	pspDebugSioInstallKprintf();
	pspDebugInstallStdoutHandler(pspDebugSioPutData);
#endif
	Kprintf("HOOK BOOT FEP enter\n");
	Kprintf("ARC  %d\n",argc);
#if 0
	for(i=0;i<argc;i++)
		Kprintf("ARG%d %s\n",i,argv[i]);
#endif

	// load current config file
	devhook_load_config(devhook_cfg_path);

#if 0
	// OLD loadexec hook reboot
	if(dh_config.reboot_path[0])
	{
		Kprintf("LoadExec old hook boot '%s'\n",dh_config.reboot_path);

		// get target path
		strcpy(target_path,dh_config.reboot_path);
		// clear reboot path
		memset(dh_config.reboot_path,0,sizeof(dh_config.reboot_path));
		devhook_save_config(devhook_cfg_path);

		if( dh_load(dh_config.flash) )
		{
Kprintf("Can't setup & load devhoook\n");
printf("Can't load devhoook\n");
	    	sceKernelExitGame();
		}
		// run target program
		load_start(target_path);
		goto exit;
	}
#endif

	// PAD init
	sceCtrlSetSamplingCycle( 0 );
	sceCtrlSetSamplingMode( 0 );
	Kprintf("pad initialized\n");

	// Screen Init
	pspDebugScreenInit();

	// default flush name
//	strcpy(dh_config.flash,"ms0:/150");

	// set default by FW1.00/1.50
	switch( sceKernelDevkitVersion())
	{
	case 0x01000300:
		boot_mode = 3; // default direct RUN
		dh_config.flash[0] =0; // Can't use reboot mode
		break;
	default: // 0x01050001
		boot_mode = 0; // default return VSH
		break;
	}

	// GUI menu
	fep_menu();

	// exit choice
	if(boot_mode < 0)
	{
		// exit to XMB
		sceKernelExitGame();
		return 0;
	}

	// Save ConfigFile for devhook.prx
	devhook_save_config(devhook_cfg_path);

	// dechook setup & load
	if( dh_load(dh_config.flash) )
	{
		sceKernelDelayThread(500000);
    	sceKernelExitGame();
	}

	// target args
	loadparam.size = sizeof(loadparam);
	loadparam.args = strlen(target_path)+1;
	loadparam.argp = target_path;
	loadparam.key  = NULL;

	// choise boot mode
	switch(boot_mode)
	{
	case 0: // return VSH(reboot)
Kprintf("sceKernelExitGame\n");
		dh_reboot_system("",REBOOT_VSH);
		//sceKernelExitGame();
		//sceKernelExitVSHVSH();
    	break;
	case 1: // boot UMDmd(reboot)
		dh_reboot_system("disc0:/PSP_GAME/SYSDIR/EBOOT.BIN",REBOOT_UMD);
//		sceKernelLoadExec("disc0:/PSP_GAME/SYSDIR/EBOOT.BIN",&loadparam);
		case 2: // boot MS
Kprintf("reboot MS %s\n",target_path);
		dh_reboot_system(target_path,REBOOT_MS);
//		sceKernelLoadExec(target_path,&loadparam);
    	break;
	default: // direct run
Kprintf("LoadStart %s\n",target_path);
		load_start(target_path);
	}
	Kprintf("Done DEVHOOK FEP\n");

//exit:

#if 0
	pspDebugInstallStdoutHandler(NULL);
	pspDebugSioDisableKprintf();
#endif

	// remove from memory
	sceKernelExitDeleteThread(sceKernelGetThreadId());

  return 0;
}
