#include "common.h"

// auto run waiting time
#define AUTORUN_TIMEOUT_SEC 3

const char umd_real_name[] = "-- UMD Disc --";
const char umd_search_dir[256] = "ms0:/ISO";
const char umd_search_dir2[256] = "ms0:/UMDEMULATOR/ISO";

/////////////////////////////////////////////////////////////////////////////
// setup 
/////////////////////////////////////////////////////////////////////////////
int boot_mode = 0;
char target_path[256];

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
int umd_lock = 0;
int top_menu_sel = 0;
int sel = 1;
int devkit;
int mjr,mnr,rev;
int speed_sel = 2;
int volt;
int umd_ver_sel;

char buf[256*32];
char *file_list[32];
char *full_list[32];

/////////////////////////////////////////////////////////////////////////////
// MENU select
/////////////////////////////////////////////////////////////////////////////
static int sel_umd(void)
{
  char *ptr;
  int sel;

  // real UMD added
  file_list[0] = (char *)umd_real_name;
  file_list[1] = 0;

  ptr = buf;

  ptr = add_file_list(umd_search_dir ,file_list,full_list,ptr,"iso");
  ptr = add_file_list(umd_search_dir2,file_list,full_list,ptr,"iso");
  ptr = add_file_list(umd_search_dir ,file_list,full_list,ptr,"cso");

  // no file ?
  if(file_list[0] == 0)
    return 0;

  // choise one
//  if(file_list[1] == 0)
//    sel = 0;
//  else
    sel = selitem(0,pspDebugScreenGetY(),(const char **)file_list,1);

	if(sel==0)
	{
		dh_config.umd_path[0] = 0; // real UMD
		dh_config.flags |= DH_FLAG_USEUMD;
		umd_lock = 2;
	}
	else  if(sel>0)
	{
		strcpy(dh_config.umd_path,full_list[sel]);
	}
  return 1;
}

const char *top_menu[] =
{
"START      ",
"UMD SELECT ",
"UMD MOUNT  ",
"UMD VERSION",
"FIRMWARE   ",
"CPU CLOCK  ",
"BOOT SELECT",
"EXIT       ",
NULL};

enum {
  TOPMENU_START,
  TOPMENU_UMD,
  TOPMENU_UMD_MOUNT,
  TOPMENU_UMD_VER,
  TOPMENU_LFLASH,
  TOPMENU_CPUCLOCK,
  TOPMENU_TARGET,
  TOPMENU_EXIT
};

const char *target_menu[] =
{
"reboot XMB             ",
"reboot UMD(1.50 only)  ",
"load run UMD(1.50 only)",
"MPHGL(1.50 only)       ",
"RunUMD(1.50 only)      ",
NULL};

const char *umd_lock_menu[] =
{
"ISO NOUMD(1.50only)",
"ISO ANYUMD         ",
"UMD DISC           ",
NULL};

const char *umd_ver_menu[] =
{
"no change   ",
"2.xx -> 2.00",
NULL};

const char *speed_menu[] =
{
" 66MHz / 37MHz",
"111MHz / 54MHz",
"222MHz /111MHz",
"333MHz /166MHz",
NULL};

const char *flash_menu[] =
{
"1.50(flash)",
"1.50       ",
"2.00       ",
"2.50       ",
//"FW2.60 (not supported)",
//"FW2.70 (not supported)",
//"FW1.00 (not supported)",
NULL};

static const char *flash_root[] =
{
	"ms0:/dh/150F",
	"ms0:/dh/150",
	"ms0:/dh/200",
	"ms0:/dh/250",
//	"ms0:/dh/260",
//	"ms0:/dh/270",
//	"ms0:/dh/100",
	NULL
};

static const char *boot_mode_str[] =
{
"reboot XMB",
"Reboot UMD",
"Reboot MS",
"Direct Run"
};

//
/*
bus : 37Mhz limit
CPU : variable
*/


/*******************************************************************************
	AUTO RUN waiting
*********************************************************************************/
static int autorun_waiting(void)
{
	int fd;

	if(!(dh_config.flags & DH_FLAG_USEUMD))
	{
		fd = sceIoOpen(dh_config.umd_path, PSP_O_RDONLY,0777);
		if(fd<0)
			return 0; // UMD file not found
		sceIoClose(fd);
	}
#if AUTORUN_TIMEOUT_SEC==0
	return 0;
#endif
	pspDebugScreenSetXY(16,6);
	printf("PUSH O to change setting\n");

	int timeout;
	for(timeout = AUTORUN_TIMEOUT_SEC*100 ; timeout >=0 ; timeout--)
	{
		if( get_key() & PSP_CTRL_CIRCLE) return 0;
		sceKernelDelayThread(10000);
	}

	pspDebugScreenSetXY(16,6);
	printf("        AUTO RUN            \n");

	return 1;
}

/*******************************************************************************
	draw display
*********************************************************************************/
void refresh_display(void)
{
	pspDebugScreenClear();

	pspDebugScreenSetXY(14,0);
	printf("PSP DEVHOOK Ver.0.41b by BOOSTER");

	// battery
	char volt_str[0x20];
	volt = scePowerGetBatteryVolt();
	switch(volt)
	{
	case 0x802b0100:
		strcpy(volt_str,"no-connect");
		break;
	default:
		if(volt>0) sprintf(volt_str,"%d.%03dV\n",volt/1000,volt%1000);
		else       sprintf(volt_str,"unknown %08X\n",volt);
	}
#if 0
  // show interface
  Kprintf("FLAGS                %08X\n",dh_config.flags);
  Kprintf("sceUmdActivate_entry %08X\n",(int)(dh_config.sceUmdActivate_entry));
  Kprintf("umd_ioctrl_table     %08X\n",(int)(dh_config.umd_ioctrl_hook_table));
  Kprintf("lflash_ioctrl_table  %08X\n",(int)(dh_config.lflash_ioctrl_orig_table));
  Kprintf("umd_path             %s\n",dh_config.umd_path);
  Kprintf("lflash_path          %s\n",dh_config.lflash_path);
  sceKernelDelayThread(2000000);
#endif

	// settings
	pspDebugScreenSetXY(0,15);
	printf("TARGET  :%s (%s)\n",target_path,boot_mode_str[boot_mode]);
	printf("UMD-ISO :%s\n",dh_config.umd_path);
	printf("UMD-MODE:%s\n",umd_lock_menu[umd_lock]);
	printf("UMD Ver.:%s\n",umd_ver_menu[umd_ver_sel]);

	printf("FLASH   :%s(%s)\n",dh_config.flash , (dh_config.flags & DH_FLAG_FLASH_EMU) ? "MS HOOK" : "NANDFlash");
	printf("CLOCK   :%03d/%03d\n",dh_config.cpu_Mhz,dh_config.bus_Mhz);
	printf("\n");
	printf("FW ver. :%02X.%02X.%04X\n",mjr,mnr,rev);
	printf("BATTERY :%s\n",volt_str);

	pspDebugScreenSetXY(0,26);
	printf("CIRCLE  :select\nTRIANGLE:cancel\nHOME+TRG_L/R:change ISO(in XMB)\n");

	//sceDisplayWaitVblankStart();
}

/*******************************************************************************
  gui main
*********************************************************************************/
int fep_menu(void)
{
	int refresh;

	devkit = sceKernelDevkitVersion();
	mjr = devkit>>24;
	mnr = (devkit>>16)&0xff;
	rev = devkit &0xffff;

	// load UMD emu mode
	umd_lock = 0;
	if(dh_config.flags & DH_FLAG_ANYUMD)
		umd_lock = 1;
	if(dh_config.flags & DH_FLAG_USEUMD)
		umd_lock = 2;

	strcpy(target_path,"");

	// SFO ver
	umd_ver_sel = (dh_config.flags & DH_FLAG_SFO_VER) ? 1 : 0;

//	strcpy(dh_config.flash,"ms0:/150");
//	dh_config.flags &= ~DH_FLAG_FLASH_EMU;

//	dh_config.cpu_Mhz = 222;
//	dh_config.bus_Mhz = 111;

	refresh_display();
	if( autorun_waiting() ) return 0;

	refresh = 1;

	while(1)
	{
		if(refresh)
		{
			refresh_display();
			refresh = 0;
		}

		pspDebugScreenSetXY(25,3);
		printf("TOP MENU");
//Kprintf("1\n");
    if(refresh<2)
    {
      sel = selitem(0,5,top_menu,top_menu_sel);
      if(sel<0) continue;
      top_menu_sel = sel;
    }
//Kprintf("2\n");
    pspDebugScreenSetXY(25,5);
    printf(top_menu[sel]);
    pspDebugScreenSetXY(0,7);
//Kprintf("3\n");
    switch(sel)
    {
    case TOPMENU_START:
      return 0;
    case TOPMENU_UMD:
//Kprintf("4\n");
//sceKernelDelayThread(1000000);
//Kprintf("5\n");
      refresh = sel_umd();
      break;
    case TOPMENU_UMD_MOUNT:
      sel = selitem(0,7,umd_lock_menu,umd_lock);
      if(sel>=0) umd_lock = sel;
      
      dh_config.flags &= ~(DH_FLAG_ANYUMD|DH_FLAG_USEUMD);
      switch(umd_lock)
      {
      case 0: break;
      case 1: dh_config.flags |= DH_FLAG_ANYUMD; break;
      case 2: dh_config.flags |= DH_FLAG_USEUMD; break;
      }
        refresh=1;
      break;
    case TOPMENU_UMD_VER:
      sel = selitem(0,7,umd_ver_menu,umd_ver_sel);
      if(sel>=0) umd_ver_sel = sel;

		if(umd_ver_sel==0)
			dh_config.flags &= ~DH_FLAG_SFO_VER;
		else
			dh_config.flags |=  DH_FLAG_SFO_VER;
		refresh=1;
      break;
 
    case TOPMENU_LFLASH:
      sel = selitem(0,7,flash_menu,0);
      if(sel>=0)
      {
		 switch(sel)
		{
		case 0:
			dh_config.flags &= ~DH_FLAG_FLASH_EMU;
			break;
		default:
			dh_config.flags |= DH_FLAG_FLASH_EMU;
			break;
		}

		// flash root dir
		strcpy(dh_config.flash,flash_root[sel]);

        refresh=1;
	}
      break;
    case TOPMENU_CPUCLOCK:
      sel = selitem(0,7,speed_menu,speed_sel);
      if(sel>=0)
      {
		int clk_val = dh_config.cpu_Mhz * 0x10000 + dh_config.bus_Mhz;
        speed_sel = sel;
        switch(sel)
        {
        case 0: clk_val =  67*0x10000 +  33; break;
        case 1: clk_val = 111*0x10000 +  55; break;
        case 2: clk_val = 222*0x10000 + 111; break;
        case 3: clk_val = 333*0x10000 + 166; break;
        }
		dh_config.cpu_Mhz = clk_val / 0x10000;
		dh_config.bus_Mhz = clk_val & 0xffff;
        refresh=1;
      }
      break;
    case TOPMENU_TARGET:
      sel = selitem(0,7,target_menu,0);
      if(sel>=0)
      {
        boot_mode = sel;
        switch(sel)
        {
        case 0: boot_mode = 0; break;
        case 1: boot_mode = 1; strcpy(target_path,"disc0:/PSP_GAME/SYSDIR/EBOOT.BIN"); break;
        case 2: boot_mode = 3; strcpy(target_path,"disc0:/PSP_GAME/SYSDIR/EBOOT.BIN"); break;

        case 3: boot_mode = 2; strcpy(target_path,"ms0:/PSP/GAME/MPHGAMELOADER/EBOOT.PBP"); break;
        case 4: boot_mode = 2; strcpy(target_path,"ms0:/PSP/GAME/RunUMD/EBOOT.PBP"); break;
 
        }
        refresh=1;
      }
      break;
    case TOPMENU_EXIT:
		boot_mode = -1;
		return 0;
    }
  }
  return 0;
}
